<?php
namespace FiloBlu\Countdown\Controller\Adminhtml\Countdown;

use Magento\Backend\App\Action;

class Save extends \Magento\Backend\App\Action
{

    protected $uploaderFactory;
    protected $imageModel;
    protected $jsHelper;
    protected $countdownFactory;
    protected $countdownTranslationsFactory;

    /**
     * @param Action\Context $context
     */
    public function __construct(
        Action\Context $context,
        \Magento\MediaStorage\Model\File\UploaderFactory $uploaderFactory,
        \FiloBlu\Countdown\Model\Countdown\Image $imageModel,
        \Magento\Backend\Helper\Js $jsHelper,
        \FiloBlu\Countdown\Model\CountdownFactory $countdownFactory,
        \FiloBlu\Countdown\Model\CountdownTranslationsFactory $countdownTranslationsFactory
    )
    {
        $this->uploaderFactory = $uploaderFactory;
        $this->imageModel = $imageModel;
        $this->jsHelper = $jsHelper;
        $this->countdownFactory = $countdownFactory;
        $this->countdownTranslationsFactory = $countdownTranslationsFactory;
        parent::__construct($context);
    }

    /**
     * {@inheritdoc}
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('FiloBlu_Countdown::config');
    }

    /**
     * Save action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        if ($data) {

            $id = $this->getRequest()->getParam('id');
            $storeId = $this->getRequest()->getParam('store',0);
            $isSavingTranslation = false;

            if ($id && $storeId != 0) {
                $isSavingTranslation = true;
            }


            $model = $this->countdownFactory->create();

            $modelTranslations = $this->countdownTranslationsFactory->create();

            if ($id) {
                $model->load($id);

                if ($isSavingTranslation) {
                    $modelTranslations->loadByParentAndStoreId($id,$storeId);
                }
            }

            $model->setData($data);

            $bgImage = $this->uploadFileAndGetName('bg_image', $this->imageModel->getBaseDir(), $data);
            $model->setBgImage($bgImage);
            
            $bgImageMobile = $this->uploadFileAndGetName('bg_image_mobile', $this->imageModel->getBaseDir(), $data);
            $model->setBgImageMobile($bgImageMobile);

            $afterBgImage = $this->uploadFileAndGetName('after_bg_image', $this->imageModel->getBaseDir(), $data);
            $model->setAfterBgImage($afterBgImage);
            
            $afterBgImageMobile = $this->uploadFileAndGetName('after_bg_image_mobile', $this->imageModel->getBaseDir(), $data);
            $model->setAfterBgImageMobile($afterBgImageMobile);

            $bgVideo = $this->uploadFileAndGetName('bg_video', $this->imageModel->getBaseDir(), $data);
            $model->setBgVideo($bgVideo);

            $afterBgVideo = $this->uploadFileAndGetName('after_bg_video', $this->imageModel->getBaseDir(), $data);
            $model->setAfterBgVideo($afterBgVideo);

            try {
                if ($isSavingTranslation) {

                    $modelTranslations->setSerializedValues(serialize($model->getData()));
                    $modelTranslations->setFilobluCountdownId($id);
                    $modelTranslations->setStoreId($storeId);
                    $modelTranslations->save();

                }
                else {
                    $this->_eventManager->dispatch(
                        'filoblu_countdown_prepare_save',
                        [
                            'countdown' => $model,
                            'request' => $this->getRequest()
                        ]
                    );

                    $model->save();
                }


                $this->messageManager->addSuccess(__('You saved this Countdown.'));

                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['id' => $model->getId() ? : $id, 'store' => $storeId, '_current' => true]);
                }

                return $resultRedirect->setPath('*/*/');
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\RuntimeException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the Countdown.'));
            }

            return $resultRedirect->setPath('*/*/edit', ['id' => $this->getRequest()->getParam('id'),'store' => $storeId]);
        }
        return $resultRedirect->setPath('*/*/');
    }

    public function uploadFileAndGetName($input, $destinationFolder, $data)
    {
        try {
            if (isset($data[$input]['delete'])) {
                return '';
            } else {
                $uploader = $this->uploaderFactory->create(['fileId' => $input]);
                $uploader->setAllowRenameFiles(true);
                $uploader->setFilesDispersion(true);
                $uploader->setAllowCreateFolders(true);
                $result = $uploader->save($destinationFolder);
                return $result['file'];
            }
        } catch (\Exception $e) {
            if ($e->getCode() != \Magento\Framework\File\Uploader::TMP_NAME_EMPTY) {
                throw new FrameworkException($e->getMessage());
            } else {
                if (isset($data[$input]['value'])) {
                    return $data[$input]['value'];
                }
            }
        }
        return '';
    }
}
